/*
    SpecRunner - Acceptance Test Driven Development Tool
    Copyright (C) 2011-2012  Thiago Santos

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
package org.specrunner.hibernate;

import java.util.Map;

import org.hibernate.event.PostInsertEvent;
import org.hibernate.event.PostInsertEventListener;
import org.hibernate.event.PreInsertEvent;
import org.hibernate.event.PreInsertEventListener;
import org.specrunner.objects.AbstractPluginObject;
import org.specrunner.util.UtilLog;

/**
 * A Hibernate listener to monitor object insertions.
 * 
 * @author Thiago Santos
 * 
 */
@SuppressWarnings("serial")
public class HibernateListener implements PreInsertEventListener, PostInsertEventListener {

    private Map<Class<?>, AbstractPluginObject> entities;
    private String keyBefore;

    public HibernateListener(Map<Class<?>, AbstractPluginObject> entities) {
        this.entities = entities;
    }

    @Override
    public boolean onPreInsert(PreInsertEvent event) {
        AbstractPluginObject pin = entities.get(event.getEntity().getClass());
        try {
            keyBefore = pin.makeKey(event.getEntity());
        } catch (Exception e) {
            if (UtilLog.LOG.isDebugEnabled()) {
                UtilLog.LOG.debug(e.getMessage(), e);
            }
        }
        return false;
    }

    @Override
    public void onPostInsert(PostInsertEvent event) {
        AbstractPluginObject pin = entities.get(event.getEntity().getClass());
        try {
            String keyAfter = pin.makeKey(event.getEntity());
            pin.mapObject(event.getEntity(), keyBefore, keyAfter);
            if (UtilLog.LOG.isInfoEnabled()) {
                UtilLog.LOG.info("HibernateListener (before=" + keyBefore + ",after=" + keyAfter + "), bound to " + event.getEntity() + ".");
            }
        } catch (Exception e) {
            if (UtilLog.LOG.isDebugEnabled()) {
                UtilLog.LOG.debug(e.getMessage(), e);
            }
        }
    }
}
